#ifndef GUI_H
#define GUI_H
//----------------------------------------------------------
#include <qpixmap.h>
//----------------------------------------------------------
#include "guibase.h"
//----------------------------------------------------------
#define PROGRAM_NAME "StructModule"
//----------------------------------------------------------
class GuiObject;
class VisualTab;
class QScrollView;
//class QMultiLineEdit;
class QTextEdit;
//----------------------------------------------------------
struct GuiObjectsList
{
	GuiObject* obj;
	GuiObjectsList* next;
};
//----------------------------------------------------------
struct GuiObjectConnectionsList
{
	GuiObject* obj1;
	GuiObject* obj2;
	GuiObjectConnectionsList* next;
};
//----------------------------------------------------------
struct StructModule;
//----------------------------------------------------------
class GUI : public GUIBase
{
	Q_OBJECT
public:
	GUI();
	~GUI();
	GuiObject* addGuiObject(int type,char* name,int x,int y);
	void addGuiObjectConnection(const char* obj1,const char* obj2);
private:
	GuiObject* findGuiObject(const char* name);
	void resizeEvent(QResizeEvent* revent);
	void mousePressEvent(QMouseEvent* pe);
	bool convertGPSSfile(const char* filename);
	void loadCode(const char* filename);
public slots:
	void updateBuffer();
	void slotGuiObjectChanged(GuiObject* obj);
	void slotActiveObjectChanged(GuiObject* obj);
	void slotActiveObjectParamsChanged(int k);
	void initWidget();
	bool init(const char* filename);
	virtual void slotStart();
	virtual void slotStop();
	virtual void slotExit();
	virtual void slotNew();
	virtual void slotOpen();
	virtual void slotClose();
	virtual void slotSaveAs();
	virtual void slotFind();
	virtual void slotPrint();
	virtual void slotAbout();
	virtual void slotAboutQt();
	virtual void slotPrepareToAddObject(int id);
	virtual void slotTabChanged(QWidget* tab);
	void slotComplete();
private:
	QSize guiObjectSize;
	QColor generatorBackgroundColor;
	QColor generatorTextColor;
	QColor queueBackgroundColor;
	QColor queueTextColor;
	QColor deviceBackgroundColor;
	QColor deviceTextColor;
	QColor packerBackgroundColor;
	QColor packerTextColor;
	QColor unpackerBackgroundColor;
	QColor unpackerTextColor;
	QColor selectorBackgroundColor;
	QColor selectorTextColor;
	QColor terminatorBackgroundColor;
	QColor terminatorTextColor;

	StructModule* module;
	QPixmap pixBuffer;
	QColor backgroundColor;
	GuiObjectsList* guiObjectsList;
	GuiObjectConnectionsList* guiObjectConnectionsList;
	VisualTab* visualTab;
	QScrollView* visualTabScroll;
	QTextEdit* codeTab;
	GuiObject* activeObject;
	GuiObject* connectionObject1;
	QTextEdit* statisticTab;
	int newObjectType;
	bool isSetConnection;
	bool isObjectMustDelete;
	
	int generatorsNum;
	int queuesNum;
	int devicesNum;
	int packersNum;
	int unpackersNum;
	int selectorsNum;
	int terminatorsNum;
signals:
	void signalSetInactive();
};
//----------------------------------------------------------
#endif
