//#include <qstring.h>
#include <qtimer.h>
#include <qfile.h>
#include <algorithm>
//----------------------------------------------------------
#include "structmodule.h"
//----------------------------------------------------------
#include "sietypes.h"
#include "siegenerator.h"
#include "sieterminator.h"
#include "siedevice.h"
#include "siequeue.h"
#include "sieunpacker.h"
#include "sieselector.h"
#include "siepacker.h"
#include "sietransactlist.h"
//----------------------------------------------------------

StructModule::StructModule(QTextEdit* outputEditBox) : modelLifeInit(-1),
			currentTime(0),workGeneratorNumInit(0),
			output(outputEditBox),transactList(0)
{
	systemTimer=new QTimer(this);
	connect(systemTimer,SIGNAL(timeout()),this,SLOT(tic()));
	transactList=new sieTransactList;

}

//----------------------------------------------------------

StructModule::~StructModule()
{
	qDebug("~StructModule");
	for(current=items.begin();current!=items.end();++current)
		delete *current;
	delete transactList;
}

//----------------------------------------------------------

sieObject* StructModule::addObject(const char* name,int type,int par1,int par2,int par3,int par4)
{
	qDebug("StructModule::add");
	sieObject* pnewitem=0;
	switch(type)
	{
	case SIE_GENERATOR:
		pnewitem=new sieGenerator(this,name,transactList,par1,par2,par3);
		connect(pnewitem,SIGNAL(signalGeneratorEmpty()),this,SLOT(slotGeneratorEmpty()));
		++workGeneratorNumInit;
		break;
		
	case SIE_TERMINATOR:
		pnewitem=new sieTerminator(this,name,transactList,par1);
		connect(pnewitem,SIGNAL(signalDecLife()),this,SLOT(slotDecLife()));
		break;
		
	case SIE_DEVICE:
		pnewitem=new sieDevice(this,name,par1,par2);
		break;
	case SIE_QUEUE:
		pnewitem=new sieQueue(this,name,par1);
		break;
	case SIE_PACKER:
		pnewitem=new siePacker(this,name,transactList,par1);
		break;
	case SIE_UNPACKER:
		pnewitem=new sieUnpacker(this,name,transactList,par1);
		break;
	case SIE_SELECTOR:
		pnewitem=new sieSelector(this,name,par1);
		break;
	default:
		qDebug("UNDEFINED TYPE!!!");
		return 0;
	}
	if(pnewitem)
	{
		items.push_back(pnewitem);
		pnewitem->setOutput(output);
	}
	return pnewitem;
}

//----------------------------------------------------------

void StructModule::setConnection(sieObject* obj1,sieObject* obj2,int type,int probability)
{
//	qDebug("setConnection");
	if(!obj1) return;
	if(!obj2) return;
	obj1->addDestination(obj2,type,probability);
	obj2->addSource(obj1,type);
}

bool StructModule::setConnection(const char* obj1name,const char* obj2name,int type,int probability)
{
	if(!obj1name) return false;
	if(!obj2name) return false;
	sieObject* obj1=find(obj1name);
	if(obj1==0) return false;
	sieObject* obj2=find(obj2name);
	if(obj2==0) return false;
	setConnection(obj1,obj2,type,probability);
	return true;
}

//----------------------------------------------------------

void StructModule::setCoordinates(const char* name,int x,int y)
{
	if(!name) return;
	sieObject* obj=find(name);
	if(!obj) return;
	obj->setCoordinates(x,y);
}

//----------------------------------------------------------

sieObject* StructModule::find(const char* name)
{
//	qDebug("StructModule::find");
	for(current=items.begin();current!=items.end();++current)
		if(strcmp(name,(*current)->name())==0)
			return *current;

	//qDebug(" result=%s",items[0]->name());
	
	return 0;//*find(items.begin(),items.end(),name);
	
	
}

//----------------------------------------------------------

void StructModule::init(bool isVisual)
{
	transactList->removeAll();

	for(current=items.begin();current!=items.end();++current)
	{
		(*current)->init();
		(*current)->setVisual(isVisual);
	}
	modelLife=modelLifeInit;
//	currentTime=0;
	workGeneratorNum=workGeneratorNumInit;
	qDebug("work generator num=%d",workGeneratorNum);
	
}

//----------------------------------------------------------

void StructModule::tic()
{
//	qDebug("StructModule::tic");
	for(current=items.begin();current!=items.end();++current)
		(*current)->tic0(currentTime);
	for(current=items.begin();current!=items.end();++current)
		(*current)->tic(currentTime);
	++currentTime;
	
	if(modelLife==0)
	{
//		qDebug("modelLife=%d",modelLife);
		stop();
		emit signalComplete();
	}
	
	if((workGeneratorNum!=0)||(!transactList->isEmpty())) 
	{
//		qDebug("%d!=0 || !%d",workGeneratorNum,transactList->isEmpty());
		return;
	}
	
	qDebug("%d ",workGeneratorNum);
	qDebug("Simulation complete!");
	
	stop();
	emit signalComplete();
}

//----------------------------------------------------------

void StructModule::slotDecLife()
{
//	qFatal("decLife");
	if(modelLife>0)
		--modelLife;
	
}

//----------------------------------------------------------

QString StructModule::getStatistic()
{
	QString str("");
	sieStatistic stat;
	for(current=items.begin();current!=items.end();++current)
	{
		(*current)->getStatistic(&stat);
		
		str+=QString(" \"%1\":\n").arg((*current)->name());
		str.append(QString(" : %1\n").arg(stat.transNum));
		if(stat.transMaxNum>0)
		{
			str.append(QString("  : %1\n").arg(stat.transMaxNum));
		}
		if(stat.transAverageAge>0)
		{
			str.append(QString("  : %1\n").arg(stat.transAverageAge));
		}
		if(stat.objectCapacity>0)
		{
			str.append(QString(": %1%\n").arg(stat.objectCapacity));
		
		}
		if(stat.sumCost>0)
		{
			str.append(QString(": $%1\n").arg(stat.sumCost));
		
		}
		str.append("\n");
	
	}

	return str;
}

//----------------------------------------------------------

void StructModule::start(int modelSpeed,int lf)
{
	for(current=items.begin();current!=items.end();++current)
		(*current)->clearStatistic();

	systemTimer->start(modelSpeed,false);
	modelLife=lf;
	currentTime=0;
	qDebug("modelLife=%d",modelLife);
}

//----------------------------------------------------------

void StructModule::stop()
{
	qDebug("\nstop\n");
	systemTimer->stop();
}

//----------------------------------------------------------

void StructModule::slotGeneratorEmpty()
{
	--workGeneratorNum;
}

